package journal.action;

import java.util.ArrayList;
import java.util.List;

import jargs.gnu.CmdLineParser;
import journal.schema.TableVersion;

public class ClientRenamer extends BaseRenamerAction
{	
	private CmdLineParser.Option oldNameOption;
	private CmdLineParser.Option newNameOption;

	public ClientRenamer() {
		super("ClientRenamer");
	}
	
	@Override
	public void start() throws Exception {
		super.start();
	}
	
	@Override
	public void help() {
		System.err.println("\t--action journal.action.ClientRenamer -- <options>");
		System.err.println("\t\tOptions are\n");
		System.err.println("\t\t-o --oldName[=]<old-name> -n --newName[=]<new-name>");
		
		super.help();
	}
	
	@Override
	public void finish() throws Exception  {
		super.finish();
	}

	@Override
	void addArgs(CmdLineParser parser) {
		oldNameOption = parser.addStringOption('o', "oldName");
		newNameOption = parser.addStringOption('n', "newName");
	}
	
	@Override
	void processArgs(CmdLineParser parser) {
        String oldName = (String) parser.getOptionValue(oldNameOption, "");
        String newName = (String) parser.getOptionValue(newNameOption, "");
        
        // verify that the correct options match up
        
        if( fileName.equals("") ) {
        	if( oldName.equals("") || newName.equals("")) {
        		throw new IllegalArgumentException("Need to specify either filename or old and new name");
        	}
        	
        	putRenamePair(oldName, newName);
        }

	}

	RenameAction getAction(TableVersion version) {
		if( !actionMap.containsKey(version)) {
			RenameAction action = EmptyAction.instance;
			
			List<RenameAction> list = new ArrayList<RenameAction>();

			if( version.getAttributeByName("client") != null ) {
				list.add( new SimpleRenameAction("client") );
			}

			if( version.getAttributeByName("cfile") != null) {
				list.add( new ClientRenameAction("cfile"));
			}
			
			if( version.getTable().getName().equals("db.view")) {
				list.add( new SimpleRenameAction("name"));
				list.add( new ClientRenameAction("vfile"));
			}
			
			if( version.getTable().getName().equals("db.domain")) {
				list.add( new SimpleRenameAction("name"));
			}
			
			if( list.size() == 1 ) {
				action = list.get(0);
			}
			else if ( list.size() > 1 ) {
				action = new CompoundActions(list);
			}
			
			actionMap.put(version, action);
		}
		
		return actionMap.get(version);
	}
}
